/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for Petit FatFs (C)ChaN, 2009      */
/*-----------------------------------------------------------------------*/

#include "diskio.h"


unsigned int readRTC(unsigned int);
void mem_cpy (void* , const void* , UINT);
DWORD bcdtod(UINT number);


extern unsigned int far drives[8];

#define DISKUNIT (*(unsigned char *)0x000320)
#define head	 (*(unsigned char *)0x000321)
#define cyl	 (*(unsigned char *)0x000322)
#define sec	 (*(unsigned char *)0x000323)
#define RESULT	 (*(unsigned char *)0x000328)
#define hstbuf	 (*(unsigned char *)0x000500)

DWORD get_fattime (void)    
{
	UINT s,m,h,dd,mm,yy;
	DWORD RESLT;

 	s=readRTC(0);
 	m=readRTC(1);
 	h=readRTC(2);  
 	dd=readRTC(3);
 	mm=readRTC(4);
 	yy=readRTC(6);  
  	

 	RESLT=(((bcdtod(yy)+20)&0x7f)<<25) + ((bcdtod(mm)&0x0f)<<21) + ((bcdtod(dd)&0x1f)<<16)+((bcdtod(h)&0x1f)<<11)+((bcdtod(m)&0x3f)<<5)+((bcdtod(s)&0x1f)/2);
	return RESLT;	
}  

DWORD bcdtod(UINT number)
{
	return ((number & 0x0f) + (((number>>4)&0x0f)*10)&0xff);
}

/*-----------------------------------------------------------------------*/
/* Initialize Disk Drive                                                 */
/*-----------------------------------------------------------------------*/

DSTATUS disk_initialize (BYTE DRIVE)
{
	DSTATUS stat;

	DISKUNIT=DRIVE;
		
	asm{
    		JSL $00FF0C 	; 
	}	

	stat=RESULT;
	return stat;
}


DSTATUS disk_status (BYTE DRIVE)
{
	DSTATUS stat;
	
	stat=0;
	// Put your code here

	return stat;
}


DRESULT disk_ioctl (BYTE DRIVE, BYTE COMMAND, void* PARM)

{
	
/* Generic command (defined for FatFs) */
//#define CTRL_SYNC			0	/* Flush disk cache (for write functions) */
//#define GET_SECTOR_COUNT	1	/* Get media size (for only f_mkfs()) */
//#define GET_SECTOR_SIZE		2	/* Get sector size (for multiple sector size (_MAX_SS >= 1024)) */
//#define GET_BLOCK_SIZE		3	/* Get erase block size (for only f_mkfs()) */
//#define CTRL_ERASE_SECTOR	4	/* Force erased a block of sectors (for only _USE_ERASE) */

/* Generic command */
//#define CTRL_POWER			5	/* Get/Set power status */
//#define CTRL_LOCK			6	/* Lock/Unlock media removal */
//#define CTRL_EJECT			7	/* Eject media */

/* MMC/SDC specific ioctl command */
//#define MMC_GET_TYPE		10	/* Get card type */
//#define MMC_GET_CSD			11	/* Get CSD */
//#define MMC_GET_CID			12	/* Get CID */
//#define MMC_GET_OCR			13	/* Get OCR */
//#define MMC_GET_SDSTAT		14	/* Get SD status */

/* ATA/CF specific ioctl command */
//#define ATA_GET_REV			20	/* Get F/W revision */
//#define ATA_GET_MODEL		21	/* Get model name */
//#define ATA_GET_SN			22	/* Get serial number */

/* NAND specific ioctl command */
//#define NAND_FORMAT			30	/* Create physical format */


	
	return RES_OK;
}

DRESULT disk_read (
	BYTE DRIVE,			/* Disk number  */
	BYTE* dest,			/* Pointer to the destination object */
	DWORD sector,			/* Sector number (LBA) */
	BYTE count			/* Sector count (bit15:destination) */
)
{

	unsigned int c,h,s,nsec,ntrack;
	DRESULT res;
	int x;

	res=0;

	if(drives[DRIVE]==0) return 1;
	DISKUNIT=DRIVE;
	if(DRIVE<4)
	{
		switch(drives[DRIVE])
		{
			case 1:
			   nsec=18;
			   ntrack=36;
			   break;
   			case 2:
			   nsec=9;
			   ntrack=18;
			   break;
   			case 3:
			   nsec=15;
			   ntrack=30;
			   break;
   			case 4:
			   nsec=9;
			   ntrack=18;
			   break;

		}
		c=sector/ntrack;
		s=(sector-(c*ntrack));
		h=0;
		if(s>=nsec) 
		{
			s=s-nsec;
			h=1;
		}

	}
	else
	{
		head=(sector>>16) & 0xff;
		cyl=(sector>>8) & 0xff;
		sec=sector & 0xff;
	}
	head=h;
	cyl=c;
	sec=s;


	asm{
    		JSL $00FF10 	; 
	}	
	mem_cpy(dest,&hstbuf,512);
	res=RESULT;
	
	return res;
	
}


  
/*-----------------------------------------------------------------------*/
/* Write Partial Sector                                                  */
/*-----------------------------------------------------------------------*/
DRESULT disk_write (
	BYTE DRIVE,			/* Disk number  */
	const BYTE* dest,		/* Pointer to the destination object */
	DWORD sector,			/* Sector number (LBA) */
	BYTE count			/* Sector count (bit15:destination) */
) 
{
	DRESULT res;
	int x;
	unsigned int c,h,s,nsec,ntrack;
	res=0;


	if(drives[DRIVE]==0) return 1;
	DISKUNIT=DRIVE;
	if(DRIVE<4)
	{
		switch(drives[DRIVE])
		{
			case 1:
			   nsec=18;
			   ntrack=36;
			   break;
   			case 2:
			   nsec=9;
			   ntrack=18;
			   break;
   			case 3:
			   nsec=15;
			   ntrack=30;
			   break;
   			case 4:
			   nsec=9;
			   ntrack=18;
			   break;

		}
		c=sector/ntrack;
		s=(sector-(c*ntrack));
		h=0;
		if(s>=nsec) 
		{
			s=s-nsec;
			h=1;
		}

	}
	else
	{
		head=(sector>>16) & 0xff;
		cyl=(sector>>8) & 0xff;
		sec=sector & 0xff;
	}
	head=h;
	cyl=c;
	sec=s;


	mem_cpy(&hstbuf, dest,512);

	asm{
    		JSL $00FF14 	; 
	}	
	res=RESULT;

	return res;
}


unsigned int readRTC(unsigned int address)
{
	unsigned int data;

	asm{
    		LDX %%address 	;    		
    		JSL $00FF1C 	; 
    		and #$00FF	;
    		STA %%data	;
	}

	return data;
}


