
;__ROMBIOS816_______________________________________________________________________________________
;
;	ROM BIOS FOR THE RBC 65c816 SBC - NATIVE MODE
;
;	WRITTEN BY: DAN WERNER -- 10/7/2017
;
;__________________________________________________________________________________________________
;
; DATA CONSTANTS
;__________________________________________________________________________________________________

		CHIP	65816		; SET CHIP		
		LONGA	OFF		; ASSUME EMULATION MODE			
		LONGI	OFF		;
		PW	128
		PL 	60
		INCLIST ON
;__________________________________________________________________________________________________
; $8000-$8007 UART 16C550 
;__________________________________________________________________________________________________
UART0:		.EQU	$FE00		;   DATA IN/OUT
UART1:		.EQU	$FE01		;   CHECK RX
UART2:		.EQU	$FE02		;   INTERRUPTS
UART3:		.EQU	$FE03		;   LINE CONTROL
UART4:		.EQU	$FE04		;   MODEM CONTROL
UART5:		.EQU	$FE05		;   LINE STATUS
UART6:		.EQU	$FE06		;   MODEM STATUS
RTC:		.EQU	$FE07		;   RTC REG.

STACK:		.EQU	$7FFF		;   POINTER TO TOP OF STACK
 
;
KEYBUFF		.EQU	$0200 		; 256 BYTE KEYBOARD BUFFER
; NATIVE VECTORS
ICOPVECTOR 	.EQU	$0300           ;COP handler indirect vector...
IBRKVECTOR 	.EQU	$0302           ;BRK handler indirect vector...
IABTVECTOR 	.EQU	$0304           ;ABT handler indirect vector...
INMIVECTOR 	.EQU	$0306           ;NMI handler indirect vector...
IIRQVECTOR 	.EQU	$0308           ;IRQ handler indirect vector...
; 6502 Emulation Vectors
IECOPVECTOR 	.EQU	$030A           ;ECOP handler indirect vector...
IEABTVECTOR 	.EQU	$030C           ;EABT handler indirect vector...
IENMIVECTOR 	.EQU	$030E           ;ENMI handler indirect vector...
IEINTVECTOR 	.EQU	$0310           ;EINT handler indirect vector...

DISKWORKAREA    .EQU	$0312		; DISK WORK AREA $0312-$031F

; DISK PARAMETER AREA
diskdevice	.EQU	$0320		; DISK DEVICE REQUESTED
debhead		.EQU	$0321		; DEBLOCKED HEAD (OR LBA)
debcyl		.EQU	$0322		; DEBLOCKED CYLINDER ID (OR LBA)
debsec		.EQU	$0323		; DEBLOCKED SECTOR (OR LBA)
Cdebhead 	.EQU	$0324		; DEBLOCKED HEAD (OR LBA) IN BUFFER
Cdebcyl		.EQU	$0325		; DEBLOCKED CYLINDER (OR LBA) ID IN BUFFER
Cdebsec		.EQU	$0326		; DEBLOCKED SECTOR (OR LBA) IN BUFFER
sekdsk		.EQU	$0327		; disk unit selected (AND IN BUFFER)
RESULT		.EQU	$0328		; disk unit RESULT CODE
hstbuf		.EQU	$0500		; word pointer to HOST BUFFER FOR DISK IO (at $0500 - $0700)



TRUE		.EQU    1
FALSE		.EQU 	0

; CHOOSE ONE CONSOLE IO DEVICE
USESERIAL 	.EQU 	TRUE	; SUPPORT FOR ON BOARD SERIAL CONSOLE IO


;USEDISKIOV1    .EQU 	FALSE	; Floppy and IDE card is  DISK IO V1
USEDISKIOV3     .EQU   	TRUE	; Floppy and IDE card is  DISK IO V3

BIOS	SECTION 	OFFSET $E000  
	;.ORG	$E000



;__COLD_START___________________________________________________
;
; PERFORM SYSTEM COLD INIT
;
;_______________________________________________________________        
COLD_START:
         	CLD			; VERIFY DECIMAL MODE IS OFF

		CLC 			; 
		XCE 			; SET NATIVE MODE
		REP #$30 		; 8 bit registers (A AND I)
		LONGI ON
		LONGA ON
		LDA #STACK 		; get the stack address
		TCS 			; and set the stack to it 

         	JSR UART_INIT		; Init UART

		
					; Announce that system is alive
         	SEP #$20 		; 8 bit accum AND INDEX
		LONGA OFF 
	        LDY   #$0000		; LOAD $00 INTO Y
OUTSTRLP:
        	LDA ALIVEM,Y   		; LOAD NEXT CHAR FROM STRING INTO ACC
        	CMP #$00		; IS NULL?
        	BEQ ENDOUTSTR		; YES, END PRINT OUT
        	JSR OUTCH  		; PRINT CHAR IN ACC
        	INY      		; Y=Y+1 (BUMP INDEX)
        	JMP OUTSTRLP		; DO NEXT CHAR
ENDOUTSTR:
         	REP #$20 		; 8 bit accum AND INDEX
		LONGA ON 


		JSR BATEST		; Perform Basic Assurance Test
		
         	LDA #INTRETURN 		;
         	STA ICOPVECTOR 
		STA IBRKVECTOR 
		STA IABTVECTOR 
		STA INMIVECTOR 
		STA IIRQVECTOR 
		STA IECOPVECTOR
		STA IEABTVECTOR
		STA IENMIVECTOR
		STA IEINTVECTOR

		REP #$30 		; 16 bit registers (A AND I)
		JSR  $9200 		; DO CMOS TEST

         	JMP  $8000		; START MONITOR


RCOPVECTOR:	JMP (ICOPVECTOR)
RBRKVECTOR:	JMP (IBRKVECTOR)
RABTVECTOR:	JMP (IABTVECTOR)
RNMIVECTOR:	JMP (INMIVECTOR)
RIRQVECTOR:	JMP (IIRQVECTOR)
RECOPVECTOR:	JMP (IECOPVECTOR)
REABTVECTOR:	JMP (IEABTVECTOR)
RENMIVECTOR:	JMP (IENMIVECTOR)
REINTVECTOR: 	JMP (IEINTVECTOR)


;__INTRETURN____________________________________________________
;
; Handle Interrupts
;
;_______________________________________________________________        
; 
INTRETURN:
		RTI 			;

;__BATEST_______________________________________________________
;
; Perform Basic Hardware Assurance Test
;
;_______________________________________________________________        
;
BATEST:
		RTS

;
;__UART_INIT____________________________________________________
;
;	INITIALIZE UART
;	PARAMS:	SER_BAUD NEEDS TO BE SET TO BAUD RATE
;	1200:	96	 = 1,843,200 / ( 16 X 1200 )
;	2400:	48	 = 1,843,200 / ( 16 X 2400 )
;	4800:	24	 = 1,843,200 / ( 16 X 4800 )
;	9600:	12	 = 1,843,200 / ( 16 X 9600 )
;	19K2:	06	 = 1,843,200 / ( 16 X 19,200 )
;	38K4:	03	
;	57K6:	02
;	115K2:	01	
;
;_______________________________________________________________
;
UART_INIT:
		SEP #$20 			; 8 bit accum (LET'S SAVE SOME ROM)
		LONGA OFF 
		
		ldx 	#63			;
		jsr 	RTC_READ		; get magic number
		cmp 	#166			; is valid?
		beq 	UART_INIT1
		LDA	#$80			;
		STA	UART3			; SET DLAB FLAG
		LDA	#12			; SET TO 12 = 9600 BAUD
		BRA 	UART_INIT2
UART_INIT1:	
		LDA	#$80			;
		STA	UART3			; SET DLAB FLAG
		ldx 	#41			;
		jsr 	RTC_READ		; get baud rate		
UART_INIT2:	
		STA	UART0			; save baud rate
		LDA	#00			;
		STA	UART1			;
		LDA	#03			;
		STA	UART3			; SET 8 BIT DATA, 1 STOPBIT
		STA	UART4			; 
		REP #$20 			; back to 16 bit mode
		LONGA ON 
		RTS

 

	IF USESERIAL=TRUE	

        	LONGA ON
        	LONGI ON
;__OUTCH_______________________________________________________
;
; OUTPUT CHAR IN LOW BYTE OF ACC TO UART
; 
;______________________________________________________________
OUTCH:	
		PHP
        	SEP #$20 			; NEED 8 bit accum for this
		LONGA OFF 
		PHA				; STORE ACC
TX_BUSYLP:
		LDA	UART5			; READ LINE STATUS REGISTER
		AND	#$20			; TEST IF UART IS READY TO SEND (BIT 5)
		CMP 	#$00
		BEQ	TX_BUSYLP		; IF NOT REPEAT
		PLA				; RESTORE ACC
		STA	UART0			; THEN WRITE THE CHAR TO UART

		PLP 				; RESTORE CPU CONTEXT
		RTS				; DONE					


        	LONGA ON
        	LONGI ON
;__INCHW_______________________________________________________
;
; INPUT CHAR FROM UART TO ACC  (WAIT FOR CHAR)
;
;______________________________________________________________
INCHW:	
		PHP
        	SEP #$20 			; NEED 8 bit accum for this
		LONGA OFF 

INCHW1:		LDA	UART5			; READ LINE STATUS REGISTER
		AND	#$01			; TEST IF DATA IN RECEIVE BUFFER
		CMP 	#$00
		BEQ	INCHW1			; LOOP UNTIL DATA IS READY
		LDA	UART0			; THEN READ THE CHAR FROM THE UART

		PLP 				; RESTORE CPU CONTEXT
		RTS


        	LONGA ON
        	LONGI ON
;__INCH_______________________________________________________
;
; INPUT CHAR FROM UART TO ACC (DO NOT WAIT FOR CHAR)
; CArry set if valid character
;______________________________________________________________
INCH:	
		PHP
        	SEP #$20 			; NEED 8 bit accum for this
		LONGA OFF 

		LDA	UART5			; READ LINE STATUS REGISTER
		AND	#$01			; TEST IF DATA IN RECEIVE BUFFER
		BEQ	INCH1			; NO CHAR FOUND
		LDA	UART0			; THEN READ THE CHAR FROM THE UART
		PLP 				; RESTORE CPU CONTEXT
		CLC
		RTS
INCH1:		PLP 				; RESTORE CPU CONTEXT
		SEC 	
		RTS

	ENDIF


;__DISPATCH_DISK_INIT__________________________________________
;
; DISPATCH DISK INIT CALL TO PROPER DEVICE
;
; REQUIRES
;  	diskdevice
;		FLOPPY 	=0-3
;		IDE 	=4-5
;		ATAPI 	=6-7
;		SD 	=8-9
;
; CARRY SET ON ERROR
;______________________________________________________________
DISPATCH_DISK_INIT:
		PHA		
		PHP
        	SEP #$30 			; NEED 8 bit REGISTERS
		LONGA OFF 
		LONGI OFF 
		
		LDA 	diskdevice
		CMP	#$04
		BMI	DISPATCH_DISK_INIT_FLOPPY
		CMP	#$06
		BMI	DISPATCH_DISK_INIT_IDE
		CMP	#$08
		BMI	DISPATCH_DISK_INIT_ATAPI
		CMP	#$0A
		BMI	DISPATCH_DISK_INIT_SD
		BRA 	DISPATCH_DISK_INIT_ERROR

DISPATCH_DISK_INIT_FLOPPY:
	IF USEDISKIOV3=TRUE
		LDA 	diskdevice
		STA 	sekdsk
		JSR 	V3FLPSETUPDRIVE
		STA 	RESULT
		PLP
		PLA
		CLC		
		RTS
	ENDIF

DISPATCH_DISK_INIT_IDE:
DISPATCH_DISK_INIT_ATAPI:
DISPATCH_DISK_INIT_SD:
DISPATCH_DISK_INIT_ERROR:
		LDA 	#$01
		STA 	RESULT
		PLP
		PLA
		SEC
		RTS


;__DISPATCH_DISK_READ__________________________________________
;
; REQUIRES
;  	diskdevice 			; DEVICE ID
;		FLOPPY 	=0-3
;		IDE 	=4-5
;		ATAPI 	=6-7
;		SD 	=8-9
;	debhead				; DEBLOCKED HEAD (OR LBA)
;	debcyl				; DEBLOCKED CYLINDER ID (OR LBA)
;	debsec				; DEBLOCKED SECTOR (OR LBA)
;
; CARRY SET ON ERROR
;______________________________________________________________
DISPATCH_DISK_READ:
		PHA		
		PHP
        	SEP #$30 			; NEED 8 bit REGISTERS
		LONGA OFF 
		LONGI OFF 
		
		LDA 	diskdevice
		CMP	#$04
		BMI	DISPATCH_DISK_READ_FLOPPY
		CMP	#$06
		BMI	DISPATCH_DISK_READ_IDE
		CMP	#$08
		BMI	DISPATCH_DISK_READ_ATAPI
		CMP	#$0A
		BMI	DISPATCH_DISK_READ_SD
		BRA 	DISPATCH_DISK_READ_ERROR

DISPATCH_DISK_READ_FLOPPY:
	IF USEDISKIOV3=TRUE
		LDA 	diskdevice
		STA 	sekdsk
		JSR 	V3FLPREADFL
		BCS 	DISPATCH_DISK_READ_ERROR
		PLP
		PLA
		CLC
		RTS
	ENDIF

DISPATCH_DISK_READ_IDE:
DISPATCH_DISK_READ_ATAPI:
DISPATCH_DISK_READ_SD:
DISPATCH_DISK_READ_ERROR:
		PLP
		PLA
		SEC
		RTS


;__DISPATCH_DISK_WRITE_________________________________________
;
; REQUIRES
;  	diskdevice 			; DEVICE ID
;		FLOPPY 	=0-3
;		IDE 	=4-5
;		ATAPI 	=6-7
;		SD 	=8-9
;	debhead				; DEBLOCKED HEAD (OR LBA)
;	debcyl				; DEBLOCKED CYLINDER ID (OR LBA)
;	debsec				; DEBLOCKED SECTOR (OR LBA)
;
; CARRY SET ON ERROR
;______________________________________________________________
DISPATCH_DISK_WRITE:
		PHA		
		PHP
        	SEP #$30 			; NEED 8 bit REGISTERS
		LONGA OFF 
		LONGI OFF 
		
		LDA 	diskdevice
		CMP	#$04
		BMI	DISPATCH_DISK_WRITE_FLOPPY
		CMP	#$06
		BMI	DISPATCH_DISK_WRITE_IDE
		CMP	#$08
		BMI	DISPATCH_DISK_WRITE_ATAPI
		CMP	#$0A
		BMI	DISPATCH_DISK_WRITE_SD
		BRA 	DISPATCH_DISK_WRITE_ERROR

DISPATCH_DISK_WRITE_FLOPPY:
	IF USEDISKIOV3=TRUE
		LDA 	diskdevice
		STA 	sekdsk
		JSR 	V3FLPWRITEFL
		BCS 	DISPATCH_DISK_WRITE_ERROR
		PLP
		PLA
		CLC
		RTS
	ENDIF

DISPATCH_DISK_WRITE_IDE:
DISPATCH_DISK_WRITE_ATAPI:
DISPATCH_DISK_WRITE_SD:
DISPATCH_DISK_WRITE_ERROR:
		PLP
		PLA
		SEC
		RTS


;______________________________________________________________
        INCLUDE 'RTC.ASM'		
;______________________________________________________________
	IF USEDISKIOV3=TRUE
	     INCLUDE 'v3FLP816.asm'
	ENDIF
;______________________________________________________________
	

	.BYTE 00,00,00		
	
ALIVEM:
                .db   $0D,$0A                                          
                .db   $0D,$0A                                         
		.db   "   __ _____  ___  __   __",$0D,$0A
		.db   "  / /| ____|/ _ \/_ | / /",$0D,$0A
		.db   " / /_| |__ | (_) || |/ /_",$0D,$0A
		.db   "|  _ \___ \ > _ < | |  _ \",$0D,$0A
		.db   "| (_) |__) | (_) || | (_) |",$0D,$0A
		.db   " \___/____/ \___/ |_|\___/ ",$0D,$0A
                .db   $0D,$0A                          
		.db   "65c816 BIOS (NATIVE MODE)",$0D,$0A
		.db   "v0.5 10/7/17 - D.WERNER",$0D,$0A
		.db   "----------------------------------------------------------------------",$0D,$0A
		.db   $0D,$0A,0
		




; BIOS JUMP TABLE (NATIVE)
		.ORG 	$FF00
LPRINTVEC	JSR	OUTCH
		RTL		
LINPVEC		JSR	INCH
		RTL		
LINPWVEC	JSR	INCHW
		RTL		
LINITDISK	JSR 	DISPATCH_DISK_INIT
		RTL		
LREADDISK	JSR  	DISPATCH_DISK_READ
		RTL		
LWRITEDISK	JSR 	DISPATCH_DISK_WRITE
		RTL		
LWRITERTC	JSR 	RTC_WRITE
		RTL		
LREADRTC	JSR 	RTC_READ
		RTL		


; BIOS JUMP TABLE (Emulation)
		.ORG 	$FF71
PRINTVEC	JMP	OUTCH		
INPVEC		JMP	INCH
INPWVEC		JMP	INCHW
INITDISK	JMP 	DISPATCH_DISK_INIT
READDISK	JMP  	DISPATCH_DISK_READ
WRITEDISK	JMP 	DISPATCH_DISK_WRITE
WRITERTC	JMP 	RTC_WRITE
READRTC		JMP 	RTC_READ

; 65c816 Native Vectors
		.ORG     $FFE4   		
COPVECTOR	.DW   RCOPVECTOR
BRKVECTOR	.DW   RBRKVECTOR
ABTVECTOR	.DW   RABTVECTOR
NMIVECTOR       .DW   RNMIVECTOR
resv1		.DW   $0000		; 
IRQVECTOR 	.DW   RIRQVECTOR	; ROM VECTOR FOR IRQ




; 6502 Emulation Vectors
		.ORG     $FFF4   		
ECOPVECTOR	.DW   RECOPVECTOR
resv2		.DW   $0000
EABTVECTOR	.DW   REABTVECTOR
ENMIVECTOR      .DW   RENMIVECTOR
RSTVECTOR       .DW   COLD_START	; 
EINTVECTOR 	.DW   REINTVECTOR	; ROM VECTOR FOR IRQ

	.END
