#include<stdio.h>
#include <stdarg.h>


int bprintf (
	const char* ,	/* Pointer to the format string */
	...					/* Optional arguments... */
);

void *heap_start = (void *) 0x3000;
void *heap_end = (void *) 0x4fff;
unsigned char nv_ram[32];
unsigned char string[100];
void writeRTC(unsigned int,unsigned int);
unsigned int readRTC(unsigned int);
void writeserial(unsigned char *);
void setupCMOS();
int readserial();
void readstring(unsigned char *);
void CMOSdrive();
void printFDriveLine(unsigned char *,int );
void printIDriveLine(unsigned char *,int );
void printSDriveLine(unsigned char *,int );
int qputc(char );
int qputs(const char *y);
void boot(void);

#define IsUpper(c)	(((c)>='A')&&((c)<='Z'))
#define IsLower(c)	(((c)>='a')&&((c)<='z'))
#define IsDigit(c)	(((c)>='0')&&((c)<='9'))


mainsetup()       
{	
	unsigned int magic,in;
	unsigned int dh,h,m,s,mm,dd,yy;	
	unsigned char apm;

	magic=readRTC(63);  // Read Magic Number

	if(magic!=166)
	{
		// must have NVRAM failure, force setup
		writeserial("NVRAM Error.\r\n\0");
		setupCMOS();
	}
	else
	{
		writeserial("NVRAM OK, Press <ctrl>-S for setup.\r\n\0");
		for(magic=0;magic<65535;magic++)
		{
			in=readserial();
			if(in==19)
			{
			    setupCMOS();
			    break;
			}
		}
	}

	apm='a';
 	s=readRTC(0);
 	m=readRTC(1);
 	h=readRTC(2);  
 	dd=readRTC(3);
 	mm=readRTC(4);
 	yy=readRTC(6);  
  	 	
 	dh=(h&15) +((h/16)*10); 
 	if(dh>12)
 	{
 		dh=dh-12;
 		apm='p';
 	}
    
 	bprintf("Current System Time is %02x/%02x/%02x, %d:%02x:%02x%cm \r\n\0",mm,dd,yy,dh,m,s,apm);
 	CMOSdrive();

 	boot();
}

void setupCMOS()
{
	int h,m,mm,dd,yy,t,dow,d0,d1,d2,d3,i0,i1,s0,s1,b0,bd;
	const int mk[13]={0,1,4,4,0,2,5,0,3,6,1,4,6};

	writeserial("Entering setup.\r\n\0");

	writeserial("Enter Current Time (HH:MM-24 hour format) ->\0");	
	readstring(string);
	sscanf(string,"%i:%i",&h,&m);
	// Encode for RTC
	t=m/10; 
	m=(m-(t*10))+(t*16);
	t=h/10;	
	h=(h-(t*10))+(t*16);

	writeserial("\n\rEnter Current Date (MM/DD/YY) ->\0");	
	readstring(string);
	sscanf(string,"%i/%i/%i",&mm,&dd,&yy);

	// determine Day of Week	
	dow=((yy/4)+dd+mk[mm]+6+yy);		  
	dow=dow-((dow/7)*7)-1;
	if(mm<3) if(yy==((yy/4)*4)) dow--;
	if(dow<0) dow=6;

	// Encode for RTC
	t=mm/10; 
	mm=(mm-(t*10))+(t*16);
	t=dd/10;	
	dd=(dd-(t*10))+(t*16);
	t=yy/10;	
	yy=(yy-(t*10))+(t*16);

	// Go ahead and set Clock
	writeRTC(7,0);
	writeRTC(2,h);
	writeRTC(1,m);
	writeRTC(0,00);

	writeRTC(3,dd);
	writeRTC(4,mm);
	writeRTC(5,dow);
	writeRTC(6,yy);

	writeRTC(8,165);  // Charge Enable
	writeRTC(63,165);  // write magic number
	writeRTC(7,128);


	writeserial("\n\rSerial Baud Rate: (Reset required before changes take effect)\n\r\0");	
	writeserial("0= 1200\n\r\0"); 
	writeserial("1= 2400\n\r\0");
	writeserial("2= 4800\n\r\0");
	writeserial("3= 9600\n\r\0");
	writeserial("4= 19K2\n\r\0");
	writeserial("5= 38K4\n\r\0");
	writeserial("6= 57K6\n\r\0");
	writeserial("7= 115K2\n\r\0");   
	writeserial("Baud Rate ->\0");	
	readstring(string);
	writeserial("\n\r\0");	
	sscanf(string,"%i",&b0);

	switch(b0)
	{
		case 0:
			b0=96;
			break;
		case 1:
			b0=48;
			break;
		case 2:
			b0=24;
			break;
		case 3:
			b0=12;
			break;
		case 4:
			b0=6;
			break;
		case 5:
			b0=3;
			break;
		case 6:
			b0=2;
			break;
		case 7:
			b0=1;
			break;
		default:
			b0=12;
			break;
	}




	
	writeserial("\n\r** Drive Setup **\n\r\0");	

	writeserial("\n\rFloppy Drive Codes:\n\r\0");	
	writeserial("0=Not In System\n\r\0");	
	writeserial("1=1.44mb\n\r\0");	
	writeserial("2=720k\n\r\0");	
	writeserial("3=1.2mb\n\r\0");	
	writeserial("4=360k\n\r\0");	
	writeserial("Drive 0 Type ->\0");	
	readstring(string);
	writeserial("\n\r\0");	
	sscanf(string,"%i",&d0);
	writeserial("Drive 1 Type ->\0");	
	readstring(string);
	writeserial("\n\r\0");	
	sscanf(string,"%i",&d1);
	writeserial("Drive 2 Type ->\0");	
	readstring(string);
	writeserial("\n\r\0");	
	sscanf(string,"%i",&d2);
	writeserial("Drive 3 Type ->\0");	
	readstring(string);
	writeserial("\n\r\0");	
	sscanf(string,"%i",&d3);

	writeserial("\n\rIDE Drive Codes:\n\r\0");	
	writeserial("0=Not In System\n\r\0");	
	writeserial("1=IDE LBA\n\r\0");	
	writeserial("2=ATAPI LBA\n\r\0");	
	writeserial("IDE Primary (Drive 4) Type ->\0");	
	readstring(string);
	writeserial("\n\r\0");	
	sscanf(string,"%i",&i0);
	writeserial("IDE Secondary (Drive 5) Type ->\0");	
	readstring(string);
	writeserial("\n\r\0");	
	sscanf(string,"%i",&i1);

	writeserial("\n\rSD Drive Codes:\n\r\0");	
	writeserial("0=Not In System\n\r\0");	
	writeserial("1=SD LBA\n\r\0");	
	writeserial("SD Primary (Drive 6) Type ->\0");	
	readstring(string);
	writeserial("\n\r\0");	
	sscanf(string,"%i",&s0);
	writeserial("SD Secondary (Drive 7) Type ->\0");	
	readstring(string);
	writeserial("\n\r\0");	
	sscanf(string,"%i",&s1);  
  
  	writeserial("Boot device unit# (99=no boot)->\0");	
	readstring(string);
	writeserial("\n\r\0");	
	sscanf(string,"%i",&bd);
	writeserial("\n\r\0");	

	// Write Config
	writeRTC(7,0);
	writeRTC(33,d0);
	writeRTC(34,d1);
	writeRTC(35,d2);
	writeRTC(36,d3);
	writeRTC(37,i0);
	writeRTC(38,i1);
	writeRTC(39,s0);
	writeRTC(40,s1);
	writeRTC(41,b0);
	writeRTC(42,bd);

	writeRTC(63,166);  // write magic number
	writeRTC(7,128);
}

void CMOSdrive()
{
	int d0,d1,d2,d3,i0,i1,s0,s1,bd;

	d0=readRTC(33);
	d1=readRTC(34);
	d2=readRTC(35);
	d3=readRTC(36);
	i0=readRTC(37);
	i1=readRTC(38);
	s0=readRTC(39);
	s1=readRTC(40);
	bd=readRTC(42);


	writeserial("\n\r** Drive Configuration **\n\r\0");	

	printFDriveLine("FD0     ",d0);	
	printFDriveLine("FD1     ",d1);	
	printFDriveLine("FD2     ",d2);	
	printFDriveLine("FD3     ",d3);	

	printIDriveLine("IDE Pri ",i0);	
	printIDriveLine("IDE Sec ",i1);	
	printSDriveLine("SD Pri  ",s0);	
	printSDriveLine("SD Sec  ",s1);	
	
	if(bd<8)
	{
		bprintf("\n\rBoot device is unit %d \n\r",bd);
	}
	else
	{
		bprintf("\n\r There is no designated boot device. \n\r",bd);
	}

}

void printFDriveLine(unsigned char *y,int d)
{
	writeserial(y);	
	switch(d)
	{
		
		case 1:
			writeserial("1.44MB\n\r\0");		
			break;
		case 2:
			writeserial("720KB\n\r\0");		
			break;
		case 3:
			writeserial("1.2MB\n\r\0");		
			break;
		case 4:
			writeserial("360KB\n\r\0");		
			break;

		default:
			writeserial("Not in System\n\r\0");		
	}
}	

void printIDriveLine(unsigned char *y,int d)
{
	writeserial(y);	
	switch(d)
	{
		
		case 1:
			writeserial("IDE LBA\n\r\0");		
			break;
		case 2:
			writeserial("ATAPI LBA\n\r\0");		
			break;
		default:
			writeserial("Not in System\n\r\0");		
	}
}	

void printSDriveLine(unsigned char *y,int d)
{
	writeserial(y);	
	switch(d)
	{
		
		case 1:
			writeserial("SD LBA\n\r\0");		
			break;
		default:
			writeserial("Not in System\n\r\0");		
	}
}	



void writeserial(unsigned char *y)
{
	unsigned char data;

	while(*y)
	{ 
	  data=*y++;

	asm{     
    		LDA %%data 	;
    		JSR $FF71 	; 
	}

	};
 
}

void readstring(unsigned char *y)
{
	int data,count;
	

	data=0;
	count=0;
	while(data!=13) {
		asm{     
    			JSR $FF77 	; 
	    		and #$00FF 	;
    			sta %%data 	;
		}

		if((data>31) && (data < 127))
		{
			count++;
			*y=data;
			*y++;
			asm{     
    				LDA %%data 	;
    				JSR $FF71 	; 
			   }
		}

		if((data==8) && (count>0))
		{
			count--;
			*y--;
			asm{     
    				LDA %%data 	;
    				JSR $FF71 	; 
    				LDA #$0020
				JSR $FF71 	; 
    				LDA %%data 	;
    				JSR $FF71 	; 
			   }
		}

	} 
	*y++=0;
 
}



int readserial()
{
	int data;
	
	asm{     
    		JSR $FF74 	; 
    		and #$00FF 	;
    		sta %%data 	;
	}

	return data;
 
}




void writeRTC(unsigned int address, unsigned int data)
{

	asm{
    		LDX %%address 	;
    		LDA %%data 	;
    		JSR $FF83 	; 
	}

}

unsigned int readRTC(unsigned int address)
{
	unsigned int data;

	asm{
    		LDX %%address 	;    		
    		JSR $FF86 	; 
    		and #$00FF	;
    		STA %%data	;
	}

	return data;
}


    
int unlink(const char *t)
{
}
int close(int t)
{
}
int read(int t)
{
}
size_t write(int x, void *y, size_t z)
{
}
long lseek(int a, long b, int c)
{
}

int isatty()
{
}



int bprintf (
	const char* str,	/* Pointer to the format string */
	...					/* Optional arguments... */
)
{
	va_list arp;
	unsigned char f, r;
	unsigned int i, j, w;
	unsigned long v;
	char c, d, s[16], *p;
	int res, chc, cc;


	va_start(arp, str);

	for (cc = res = 0; cc != EOF; res += cc) {
		c = *str++;
		if (c == 0) break;			/* End of string */
		if (c != '%') {				/* Non escape character */
			cc = qputc(c);  
			if (cc != EOF) cc = 1;
			continue;
		}
		w = f = 0;
		c = *str++;
		if (c == '0') {				/* Flag: '0' padding */
			f = 1; c = *str++;
		} else {
			if (c == '-') {			/* Flag: left justified */
				f = 2; c = *str++;
			}
		}
		while (IsDigit(c)) {		/* Precision */
			w = w * 10 + c - '0';
			c = *str++;
		}
		if (c == 'l' || c == 'L') {	/* Prefix: Size is long int */
			f |= 4; c = *str++;
		}
		if (!c) break;
		d = c;
		if (IsLower(d)) d -= 0x20;
		switch (d) {				/* Type is... */
		case 'S' :					/* String */
			p = va_arg(arp, char*);
			for (j = 0; p[j]; j++) ;
			chc = 0;
			if (!(f & 2)) {
				while (j++ < w) chc += (cc = qputc(' '));
			}
			chc += (cc = qputs(p));
			while (j++ < w) chc += (cc = qputc(' '));
			if (cc != EOF) cc = chc;
			continue;
		case 'C' :					/* Character */
			cc = qputc((char)va_arg(arp, int)); continue;
		case 'B' :					/* Binary */
			r = 2; break;
		case 'O' :					/* Octal */
			r = 8; break;
		case 'D' :					/* Signed decimal */
		case 'U' :					/* Unsigned decimal */
			r = 10; break;
		case 'X' :					/* Hexdecimal */
			r = 16; break;
		default:					/* Unknown type (passthrough) */
			cc = qputc(c); continue;
		}

		/* Get an argument and put it in numeral */
		v = (f & 4) ? (unsigned long)va_arg(arp, long) : ((d == 'D') ? (unsigned long)(long)va_arg(arp, int) : (unsigned long)va_arg(arp, unsigned int));
		if (d == 'D' && (v & 0x80000000)) {
			v = 0 - v;
			f |= 8;
		}
		i = 0;
		do {
			d = (char)(v % r); v /= r;
			if (d > 9) d += (c == 'x') ? 0x27 : 0x07;
			s[i++] = d + '0';
		} while (v && i < sizeof(s) / sizeof(s[0]));
		if (f & 8) s[i++] = '-';
		j = i; d = (f & 1) ? '0' : ' ';
		res = 0;
		while (!(f & 2) && j++ < w) res += (cc = qputc(d));
		do res += (cc = qputc(s[--i])); while(i);
		while (j++ < w) res += (cc = qputc(' '));
		if (cc != EOF) cc = res;
	}

	va_end(arp);
	return (cc == EOF) ? cc : res;
}

int qputc(char c)
{
	char data;
	  data=c;
  
	asm{     
    		LDA %%data 	;
    		JSL $00FF00 	; 
	}

	return c;
 
}

int qputs(const char *y)
{
	char data;

	while(*y)
	{ 
	  data=*y++;

	asm{     
    		LDA %%data 	;
    		JSL $00FF00 	; 
	}

	}; 
	return 1;
}


