
;__RTC__________________________________________________________________________________________
;
;	DS1302 DRIVERS FOR THE RBC 65c816 SBC
;
;	WRITTEN BY: DAN WERNER -- 10/8/2017
;
;__________________________________________________________________________________________________
;
; DATA CONSTANTS
;__________________________________________________________________________________________________

		CHIP	65816		; SET CHIP		
		LONGA	OFF		; ASSUME EMULATION MODE			
		LONGI	OFF		;
		PW	128
		PL 	60
		INCLIST ON


;__________________________________________________________________________________________________
; $80A0-$80A3 FRONT PANEL 8255
;__________________________________________________________________________________________________



; TEMP STUFF

PRINTVEC    .EQU    $FF71
INPVEC      .EQU    $FF74
INPWVEC     .EQU    $FF77
INITDISK    .EQU    $FF7A
READDISK    .EQU    $FF7D 
WRITEDISK   .EQU    $FF80
RTC_WRITE   .EQU    $FF83
RTC_READ    .EQU    $FF86



	.ORG	4000H



        REP #$30            ; 8 bit registers (A AND I)
        LONGI ON
        LONGA ON

        SEP #$20            ; 8 bit accum 
        LONGA OFF 
        LDY   #$0000        ; LOAD $00 INTO Y
OUTSTRLP:
        LDA HELLO,Y     ; LOAD NEXT CHAR FROM STRING INTO ACC
        CMP #$00        ; IS NULL?
        BEQ ENDOUTSTR   ; YES, END PRINT OUT
        JSR PRINTVEC    ; PRINT CHAR IN ACC
        INY             ; Y=Y+1 (BUMP INDEX)
        JMP OUTSTRLP    ; DO NEXT CHAR
ENDOUTSTR:

        JSR RTC_CHARGE_ENABLE
;       JSR RTC_SET_NOW

        LDX #$0000
        JSR RTC_READ
        STA $5000
        LDX #$0001
        JSR RTC_READ
        STA $5001
        LDX #$0002
        JSR RTC_READ
        STA $5002
        LDX #$0003
        JSR RTC_READ
        STA $5003
        LDX #$0004
        JSR RTC_READ
        STA $5004
        LDX #$0005
        JSR RTC_READ
        STA $5005
        LDX #$0006
        JSR RTC_READ
        STA $5006
        LDX #$0007
        JSR RTC_READ
        STA $5007


        REP #$20        ; 16 bit accum 
        LONGA ON 

        BRK




; function RTC_WR_PROTECT

RTC_WR_PROTECT:
    PHA
    PHX
    PHP 
    SEP #$30            ; 8 bit REGISTERS
    LONGA OFF 
    LONGI OFF 
    LDX #%00000111
    LDA #%10000000
    JSR RTC_WRITE
    PLP
    PLX
    PLA
    RTS


; function RTC_WR_UNPROTECT

RTC_WR_UNPROTECT:
    PHA
    PHX
    PHP 
    SEP #$30            ; 8 bit REGISTERS
    LONGA OFF 
    LONGI OFF 
    LDX #%00000111
    LDA #%00000000
    JSR RTC_WRITE
    PLP
    PLX
    PLA
    RTS




RTC_SET_NOW:
    php
    sEP #$30            ; 8 bit registers (A AND I)
    LONGI Off
    LONGA Off

; set time to 07-02-23 19:45:00-05 <-Friday
    JSR RTC_WR_UNPROTECT
; seconds
    LDX  #$00
    LDA  #$00
    JSR RTC_WRITE

; minutes
    LDX  #$01
    LDA  #$45
    JSR RTC_WRITE

; hours
    LDX  #$02
    LDA  #$19
    JSR RTC_WRITE

; day
    LDX  #$03
    LDA  #$23
    JSR RTC_WRITE

; month
    LDX  #$04
    LDA  #$02
    JSR RTC_WRITE

; dow
    LDX  #$05
    LDA  #$06
    JSR RTC_WRITE

; year
    LDX  #$06
    LDA  #$07
    JSR RTC_WRITE

    JSR RTC_WR_PROTECT
    plp
    RTS


; function RTC_RESTART
RTC_RESTART:
    php
    sEP #$30            ; 8 bit registers (A AND I)
    LONGI Off
    LONGA Off
    JSR RTC_WR_UNPROTECT
    LDX  #$00
    LDA  #$00
    JSR RTC_WRITE
    JSR RTC_WR_PROTECT
    plp 
    RTS


; function RTC_CHARGE_ENABLE
RTC_CHARGE_ENABLE
    php
    sEP #$30            ; 8 bit registers (A AND I)
    LONGI Off
    LONGA Off

    JSR RTC_WR_UNPROTECT
    LDX #$08
    LDA #$A5
    JSR RTC_WRITE
    JSR RTC_WR_PROTECT
    plp
    RTS


; function RTC_CHARGE_DISABLE

RTC_CHARGE_DISABLE
    php
    sEP #$30            ; 8 bit registers (A AND I)
    LONGI Off
    LONGA Off

    JSR RTC_WR_UNPROTECT
    LDX #$08
    LDA #$00
    JSR RTC_WRITE
    JSR RTC_WR_PROTECT
    plp
    RTS





;_________________________________________________________________________________________________________________________

;_Text Strings____________________________________________________________________________________________________________
; 
HELLO:
    .DB $0A, $0D     ; line feed and carriage return
    .DB $0A, $0D     ; line feed and carriage return
	.DB	"Begin RTC Test Program"
	.DB	$0A, $0D,00		; line feed and carriage return
;_________________________________________________________________________________________________________________________


.end


